@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 """
-Copyright (c) 2020, ETH Zurich, Computer Engineering Group (TEC)
+Copyright (c) 2021, ETH Zurich, Computer Engineering Group (TEC)
 """
 
 import base64
@@ -19,6 +19,10 @@ import numpy as np
 import pandas as pd
 import appdirs
 from getpass import getpass
+from xml.etree import ElementTree as et
+import io
+from elftools.elf.elffile import ELFFile   # requires package "pyelftools"
+import struct
 
 ################################################################################
 
@@ -262,10 +266,11 @@ class Flocklab:
             print(e)
             print("ERROR: Failed to contact the FlockLab API!")
 
-    def getResults(self, testId):
+    def getResults(self, testId, outDir='./', extract=True):
         '''Download FlockLab test results via https.
         Args:
             testId: ID of the test which should be downloaded
+            outDir: Download directory (default: current working path)
         Returns:
             Success of download as string.
         '''
@@ -299,15 +304,18 @@ class Flocklab:
                 output = json.loads(req.text)["output"]
                 raise FlocklabError('FlockLab API Error: {}'.format(output))
             elif 'application/x-gzip' in req.headers['content-type']:
-                with open('flocklab_testresults_{}.tar.gz'.format(testId), 'wb') as f:
+                with open(os.path.join(outDir, 'flocklab_testresults_{}.tar.gz'.format(testId)), 'wb') as f:
                     f.write(req.content)
             else:
                 raise FlocklabError('Server response contains unexpected response content-type: {}'.format(req.headers['content-type']))
 
-            print("extracting archive ...")
-            with tarfile.open('flocklab_testresults_{}.tar.gz'.format(testId)) as tar:
-                tar.extractall()
-            return 'Successfully downloaded & extracted: flocklab_testresults_{}.tar.gz & {}'.format(testId, testId)
+            if extract:
+                print("extracting archive ...")
+                with tarfile.open(os.path.join(outDir, 'flocklab_testresults_{}.tar.gz'.format(testId))) as tar:
+                    tar.extractall(path=outDir)
+                return 'Successfully downloaded & extracted: flocklab_testresults_{}.tar.gz & {}'.format(testId, testId)
+            else:
+                return 'Successfully downloaded flocklab_testresults_{}.tar.gz'.format(testId)
 
     def getTestInfo(self, testId):
         '''Get information for an existing FlockLab test.
@@ -400,44 +408,210 @@ class Flocklab:
 
 
     @staticmethod
-    def serial2Df(serialPath, error='replace', serialFilename='serial.csv'):
+    def serial2Df(serialFile, error='replace', fileName='serial.csv'):
         '''Read a serial trace from a flocklab test result and convert it to a pandas dataframe.
         Args:
-            serialPath: path to serial trace result file (or flocklab result directory)
+            serialFile: File path or file pointer (binary read mode, 'rb') to serial logging file, or path to FockLab result directory containing the serial logging file
+            error:      How to handle binary to string decoding errors
+            fileName:   File name of the FlockLab serial logging file; used in case a string pointing to a test result directory is passed to serialFile (default: serial.csv)
         Returns:
             serial log as pandas dataframe
         '''
-        if os.path.splitext(serialFilename)[1] != '.csv':
-            if os.path.isdir(serialPath):
-                serialPath = os.path.join(serialPath, serialFilename)
+        f = None
+        if type(serialFile)==str:
+            if os.path.isdir(serialFile):
+                serialFile = os.path.join(serialFile, fileName)
+            if not os.path.isfile(serialFile):
+                raise Exception('The file does not exist: {}'.format(serialFile))
+            f = open(serialFile, 'rb')
+        else:
+            f = serialFile
+
+        buf = f.read()
+        # replace carriage returns (e.g. contained in corrupted radio messages) as they are interpreted as newline by readlines(), flocklab serial output does never contain carriage return for line break as flocklab converts all CRLF into LF (\r\n -> \n)
+        buf = buf.replace(bytes(u'\r', encoding='utf8'), bytes(u'', encoding='utf8')) # removes all carriage returns ('\r' or u'\u000d')
+        f2 = io.StringIO(buf.decode(encoding='utf-8', errors='replace'))
+        # read data into dataframe
+        ll = []
+        header_processed = False
+        for line in f2.readlines():
+            if not header_processed:
+                cols = line.rstrip().split(',')
+                assert len(cols) == 5
+                header_processed = True
+                continue
+            parts = line.rstrip().split(',')
+            if len(parts) >= 5:
+                ll.append(OrderedDict([
+                  (cols[0], float(parts[0])),          # timestamp
+                  (cols[1], int(parts[1])),            # observer_id
+                  (cols[2], int(parts[2])),            # node_id
+                  (cols[3], parts[3]),                 # direction
+                  (cols[4], ','.join(parts[4:])),      # output
+                ]))
             else:
-                raise RuntimeError('The provided path is not valid: %s' % serialPath)
+                raise Exception('ERROR: line does not contain enough columns: {}'.format(line))
 
-        if not os.path.isfile(serialPath):
-            raise RuntimeError('The file does not exist: %s' % serialFilename)
+        if type(serialFile)==str:
+            f.close()
 
-        with open(serialPath, 'r', encoding='utf-8', errors='replace') as f:
-            ll = []
-            header_processed = False
-            for line in f.readlines():
-                if not header_processed:
-                    cols = line.rstrip().split(',')
-                    assert len(cols) == 5
-                    header_processed = True
-                    continue
-                parts = line.rstrip().split(',')
-                if len(parts) > 0:
-                    ll.append(OrderedDict([
-                      (cols[0], parts[0]),                  # timestamp
-                      (cols[1], int(parts[1])),             # observer_id
-                      (cols[2], int(parts[2])),            # node_id
-                      (cols[3], parts[3]),                 # direction
-                      (cols[4], ','.join(parts[4:])),      # output
-                    ]))
         df = pd.DataFrame.from_dict(ll)
         df.columns
         return df
 
+    @staticmethod
+    def getCustomField(testConfigFile, testConfigFileName='testconfig.xml'):
+        '''Tries to read info from xml field `custom`.
+        Args:
+            testConfigFile:      File path as string or file pointer of the FlockLab test config file, or file path (string) of a FlockLab test result directory
+            testConfigFileName:  File name of the FlockLab test config file; used in case a string pointing to a test result directory is passed to testConfigFile (default: testconfig.xml)
+        Returns:
+            Content of custom field in testconfig.xml as string.
+        '''
+        if type(testConfigFile)==str and os.path.isdir(testConfigFile):
+            testConfigFile = os.path.join(testConfigFile, testConfigFileName)
+
+        try:
+            # read custom field in testconfig xml
+            ns = {'dummy': 'http://www.flocklab.ethz.ch'}
+            tree = et.parse(testConfigFile)
+            ret = tree.findall('.//dummy:custom', namespaces=ns)
+            assert len(ret) == 1
+        except Exception as e:
+            return None
+        else:
+            return ret[0].text
+
+    @staticmethod
+    def getDtAddrToVarMap(testConfigFile, testConfigFileName='testconfig.xml'):
+        '''Tries to read datatrace specific info from xml field `custom`. Always returns a dict (even if empty).
+        Args:
+            testConfigFile:      File path as string or file pointer of the FlockLab test config file, or file path (string) of a FlockLab test result directory
+            testConfigFileName:  File name of the FlockLab test config file; used in case a string pointing to a test result directory is passed to testConfigFile (default: testconfig.xml)
+        Returns:
+            Dictionary with mapping of datatrace addresses to variables
+        '''
+        try:
+            custom = Flocklab.getCustomField(testConfigFile, testConfigFileName)
+            d = json.loads(custom)
+            d = d['datatrace']['var_to_addr_map']
+            # reverse dict
+            d = {v:k for k,v in d.items()}
+        except Exception as e:
+            # we ignore any error since info is not essential, nevertheless it is important to return a dict() such that the mapping does not fail
+            return dict()
+        else:
+            return d
+
+    @staticmethod
+    def structSizeMap(numBytes):
+        if numBytes == 1: return 'B'
+        elif numBytes == 2: return 'H'
+        elif numBytes == 4: return 'I'
+        elif numBytes == 8: return 'Q'
+        else: raise Exception('Undefined number of bytes!')
+
+    @staticmethod
+    def getSymbolAddress(elfPath, symbName):
+        '''Get the address of the symbol with name symbName from ELF file elfPath.
+        Args:
+            elfPath:  Path to the elf file
+            symbName: Name of the symbol
+        Returns:
+            Address
+        '''
+        with open(elfPath, 'rb') as f:
+            elf = ELFFile(f)
+            symtab = elf.get_section_by_name('.symtab')
+            if not symtab:
+                raise Exception('ELF file does not contain a symbol table!')
+            sym = symtab.get_symbol_by_name(varname)
+
+            if sym is None or len(sym) == 0:
+                raise Exception('Symbol "{}" not found!'.format(symbName))
+            elif len(sym) > 1:
+                raise Exception('Found multiple entries for Symbol "{}"!'.format(symbName))
+            return sym[0]['st_value']
+
+    @staticmethod
+    def getSymbolFileOffsetAndSize(elfPath, symbName):
+        '''Get the file offset (in bytes) and the size of the symbol with name symbName from ELF file elfPath.
+        Args:
+            elfPath:  Path to the elf file
+            symbName: Name of the symbol
+        Returns:
+            File offset (in bytes)
+        '''
+        fileOffset = None
+        symbSize = None
+
+        with open(elfPath, 'rb') as f:
+            elf = ELFFile(f)
+
+            # read symbol table
+            symtab = elf.get_section_by_name('.symtab')
+            if not symtab:
+                raise Exception('ELF file does not contain a symbol table!')
+
+            # get symbol & symbol size
+            symList = symtab.get_symbol_by_name(symbName)
+            if not symList:
+                raise Exception('Symbol "{}" not found!'.format(symbName))
+            sym = symList[0]
+            symbAddr = sym['st_value']
+            symbSize = sym['st_size']
+
+            # get file offset
+            fileOffset = next(elf.address_offsets(symbAddr), None)
+
+            if not fileOffset:
+                raise Exception('Could not determine file offset!')
+
+        return fileOffset, symbSize
+
+    @staticmethod
+    def readSymbolValue(elfPath, symbName, signed=False):
+        '''Read value of symbol symbName from elf file elfPath.
+        Args:
+            elfPath:  Path to the elf file in which the symbol will be replaced
+            symbName: Name of the symbol
+            signed:   If True, the symbol is assumed to be a signed integer, otherwise the symbol is assumed to be an unsigned integer (default: False)
+        Returns:
+            Value of symbol
+        '''
+        value = None
+
+        with open(elfPath, 'rb') as f:
+            elf = ELFFile(f)
+            fileOffset, symbSize = Flocklab.getSymbolFileOffsetAndSize(elfPath, symbName)
+            elf.stream.seek(fileOffset)
+            if signed:
+                value = struct.unpack(Flocklab.structSizeMap(symbSize).lower(), elf.stream.read(symbSize))[0]
+            else:
+                value = struct.unpack(Flocklab.structSizeMap(symbSize), elf.stream.read(symbSize))[0]
+
+        return value
+
+    @staticmethod
+    def writeSymbolValue(elfPath, symbName, symbReplace, signed=False):
+        '''Searches symbol symbName in ELF file elfPath and replaces its value with symbReplace.
+        Args:
+            elfPath:        Path to the elf file in which the symbol will be replaced
+            symbName:       Name of the symbol whose value will be replaced
+            symbReplace:    Replacement value (int)
+            signed:         If True, the symbol is assumed to be a signed integer, otherwise the symbol is assumed to be an unsigned integer (default: False)
+        '''
+        symbFileOffset, symbSize = Flocklab.getSymbolFileOffsetAndSize(elfPath, symbName)
+
+        with open(elfPath, 'rb+') as f:
+            f.seek(symbFileOffset)
+            if signed:
+                replaceBytes = struct.pack(Flocklab.structSizeMap(symbSize).lower(), symbReplace)
+            else:
+                replaceBytes = struct.pack(Flocklab.structSizeMap(symbSize), symbReplace)
+            f.write(replaceBytes)
+            f.close()
+
 
 ################################################################################
 
