@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 """
-Copyright (c) 2020, ETH Zurich, Computer Engineering Group (TEC)
+Copyright (c) 2021, ETH Zurich, Computer Engineering Group (TEC)
 """
 
 import numpy as np
@@ -11,8 +11,8 @@ import os
 import sys
 import glob
 from copy import copy
-from xml.etree import ElementTree as et
 import json
+from rocketlogger.data import RocketLoggerData
 
 from bokeh.plotting import figure, show, save, output_file
 from bokeh.models import ColumnDataSource, Plot, Span, BoxAnnotation, CrosshairTool, HoverTool, CustomJS, Div, Select, CheckboxButtonGroup, CustomJSHover
@@ -24,7 +24,13 @@ from bokeh.colors.named import red, green, blue, orange, lightskyblue, mediumpur
 from bokeh.events import Tap, DoubleTap, ButtonClick
 
 from .flocklab import FlocklabError
+from flocklab import Flocklab
+fl = Flocklab()
 
+# ignore bokeh deprecation warning until color.darken in bokeh gets fixed
+import warnings
+from bokeh.util.warnings import BokehDeprecationWarning
+warnings.filterwarnings("ignore", category=BokehDeprecationWarning)
 
 ###############################################################################
 
@@ -94,27 +100,7 @@ def trace2series(t, v):
 
     return (tNewNew, vNewNew)
 
-def get_dt_addr_to_var_map(resultPath):
-    '''Tries to read datatrace specific info from xml field `custom`. Always returns a dict (even if empty).
-    '''
-    try:
-        # read custom field in testconfig xml
-        ns = {'dummy': 'http://www.flocklab.ethz.ch'}
-        tree = et.parse(os.path.join(resultPath, 'testconfig.xml'))
-        ret = tree.findall('.//dummy:custom', namespaces=ns)
-        assert len(ret) == 1
-        d = json.loads(ret[0].text)
-        d = d['datatrace']['var_to_addr_map']
-        # reverse dict
-        d = {v:k for k,v in d.items()}
-    except Exception as e:
-        # we ignore any error since info is not essential, nevertheless it is important to return a dict() such that the mapping does not fail
-        return dict()
-    else:
-        return d
-
 def plotObserverGpio(nodeId, nodeData, prevPlot, absoluteTimeFormatter):
-    colors = ['blue', 'red', 'green', 'orange']
     p = figure(
         title=None,
         x_range=prevPlot.x_range if prevPlot is not None else None,
@@ -329,6 +315,7 @@ def plotAll(gpioData, powerData, datatraceData, testNum, absoluteTimeFormatter,
     timePlot.add_glyph(source, vareaGlyph)
     timePlot.xgrid.grid_line_color = None
     timePlot.ygrid.grid_line_color = None
+    timePlot.xaxis.major_label_text_font_size = '10pt'
     timePlot.yaxis.visible = False
     allPlots += [timePlot]
 
@@ -437,6 +424,7 @@ def createAppAndRender(gpioPlots, powerPlots, datatracePlots, timePlot, testNum,
                 'width': '30px',
                 'height': '100%',
                 'text-align': 'center',
+                # 'font-size': '18px',
             },
             # sizing_mode='stretch_both',
             align='center',
@@ -880,7 +868,6 @@ def visualizeFlocklabTrace(resultPath, outputDir=None, interactive=False, showPp
         interactive: switch to turn on/off automatic display of generated bokeh plot
     '''
     # check if resultPath is not empty
-    print(resultPath)
     if resultPath.strip() == '' or resultPath is None:
         raise Exception('ERROR: No FlockLab result directory provided as argument!')
 
@@ -932,8 +919,6 @@ def visualizeFlocklabTrace(resultPath, outputDir=None, interactive=False, showPp
 
             powerAvailable = True
     elif powerRldFiles:
-        from rocketlogger.data import RocketLoggerData
-
         powerDfList = []
         for powerRldFile in powerRldFiles:
             sp = os.path.basename(powerRldFile).split('.')
@@ -942,8 +927,9 @@ def visualizeFlocklabTrace(resultPath, outputDir=None, interactive=False, showPp
             tempDf = pd.DataFrame()
             rld = RocketLoggerData(powerRldFile)
             rld.merge_channels()
-            ts = rld.get_time(absolute_time=True, time_reference='network')
-            tempDf['timestamp'] = ts.astype('uint64') / 1e9   # convert to s
+            ts = rld.get_time(time_reference='network')
+            tempDf['timestamp'] = ts.astype(np.timedelta64) / np.timedelta64(1, 's')   # convert numpy.datetime64 timestamps into unix timestamps (numpy.float64 in seconds)
+            # tempDf['timestamp'] = ts.astype('uint64') / 1e9   # convert numpy.datetime64 timestamps into unix timestamps (numpy.float64 in seconds)
             tempDf['observer_id'] = obsId
             tempDf['node_id'] = nodeId
             tempDf['current_mA'] = rld.get_data('I1') * 1e3 # convert to mA
@@ -1073,8 +1059,8 @@ def visualizeFlocklabTrace(resultPath, outputDir=None, interactive=False, showPp
                   'access': variableGrp['access'].to_numpy(),
                   'delay_marker': variableGrp['delay_marker'].to_numpy(),
                 }
-                addr_to_var_map = get_dt_addr_to_var_map(resultPath=resultPath)
-                variableNameMapped = addr_to_var_map[variableName] if variableName in addr_to_var_map else variableName
+                addrToVarMap = fl.getDtAddrToVarMap(testConfigFile=resultPath)
+                variableNameMapped = addrToVarMap[variableName] if variableName in addrToVarMap else variableName
                 nodeData.update({variableNameMapped: trace})
             datatraceData.update({nodeId: nodeData})
 
