@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 """
-Copyright (c) 2020, ETH Zurich, Computer Engineering Group (TEC)
+Copyright (c) 2021, ETH Zurich, Computer Engineering Group (TEC)
 """
 
 import numpy as np
@@ -81,7 +81,7 @@ class FlocklabXmlConfig():
                 else:
                     print('Image last modified: {}'.format(datetime.datetime.fromtimestamp(os.path.getmtime(config.imagePath))))
 
-        print('Used nodes (all): {}'.format(list(set(allNodes))))
+        print('Used nodes (all): {}'.format(sorted(list(set(allNodes)))))
 
         # concatenate config sections
         for config in self.configList:
@@ -173,8 +173,6 @@ class SerialConf():
     def config2Et(self, x):
         if self.obsIds is None:
             raise Exception('ERROR: obsIds of SerialConf needs to be set!')
-        if self.remoteIp is None:
-            self.remoteIp = '0.0.0.0' # useless for forwarding but prevents problems with gpiotracing if by default an IPv6 address is set
 
         x.append(et.Comment(FlocklabXmlConfig.extendTitle('Serial Configuration')))
         sc = FlocklabXmlConfig.addSubElement(x, 'serialConf')
@@ -189,7 +187,8 @@ class SerialConf():
                 pass
             else:
                 raise Exception('ERROR: Invalid serial config!')
-        FlocklabXmlConfig.addSubElement(sc, 'remoteIp', text=self.remoteIp)
+        if self.remoteIp is not None:
+            FlocklabXmlConfig.addSubElement(sc, 'remoteIp', text=self.remoteIp)
         return x
 
 class GpioTracingConf():
@@ -228,7 +227,10 @@ class GpioActuationConf():
             if type(pinConf) != dict:
                 raise Exception('ERROR: element of pinConfList must be of type \'dict\'!')
             if len(set(pinConf.keys()).intersection(set(['pin', 'level', 'offset']))) != 3:
-                raise Exception('ERROR: pinConfList must be a dict containing items \'pin\', \'level\', and \'offset\' values!')
+                raise Exception('ERROR: pinConfList item must be a dict containing items \'pin\', \'level\', and \'offset\'!')
+            remainingKeys = set(pinConf.keys()).difference(set(['pin', 'level', 'offset']))
+            if len(remainingKeys) and len(remainingKeys.intersection(set(['period', 'count']))) != 2:
+                raise Exception('ERROR: pinConfList item may optionally contain items \'period\' AND \'count\'!')
 
         x.append(et.Comment(FlocklabXmlConfig.extendTitle('GPIO Actuation Configuration')))
         gtc = FlocklabXmlConfig.addSubElement(x, 'gpioActuationConf')
@@ -242,6 +244,9 @@ class GpioActuationConf():
             FlocklabXmlConfig.addSubElement(pinConfXml, 'pin', text='{}'.format(pinConf['pin']))
             FlocklabXmlConfig.addSubElement(pinConfXml, 'level', text='{}'.format(pinConf['level']))
             FlocklabXmlConfig.addSubElement(pinConfXml, 'offset', text='{}'.format(pinConf['offset']))
+            if len(pinConf) == 5:
+                FlocklabXmlConfig.addSubElement(pinConfXml, 'period', text='{}'.format(pinConf['period']))
+                FlocklabXmlConfig.addSubElement(pinConfXml, 'count', text='{}'.format(pinConf['count']))
         return x
 
 
